<!DOCTYPE html>
<html lang="en" data-theme="dark">
  <head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <title>llmpm — {{MODEL_ID}}</title>
    <link rel="preconnect" href="https://fonts.googleapis.com" />
    <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin="anonymous" />
    <link href="https://fonts.googleapis.com/css2?family=Press+Start+2P&family=Space+Grotesk:wght@300;400;500;600;700&family=JetBrains+Mono:wght@400;500&display=swap" rel="stylesheet" />
    <!--
      The Python server template-replaces {{MODEL_ID}} and {{MODEL_CATEGORY}}
      before serving this file. These are NOT Vite template variables — Vite
      will leave inline <script> content untouched.
    -->
    <script>
      window.LLMPM_CONFIG = {
        modelId:  "{{MODEL_ID}}",
        category: "{{MODEL_CATEGORY}}",
        models:   {{MODELS_JSON}}
      };
    </script>
    <script type="module" crossorigin src="/static/assets/index-BSk9mF4T.js"></script>
    <link rel="stylesheet" crossorigin href="/static/assets/index-DzypsRmw.css">
  </head>
  <body>
    <div id="root"></div>
  </body>
</html>
